@@ -8,7 +8,7 @@
 #'                   chunk_size=1e7, remove.intercept=NULL,remove.unused.columns=TRUE)
 #'        
 #'        create.dml(ddl,model.parameters,design.parameters,restrict=FALSE,
-#'              chunk_size=1e7,use.admb=FALSE,remove.unused.columns=TRUE)
+#'              chunk_size=1e7,use.admb=FALSE,remove.unused.columns=TRUE,simplify=FALSE)
 #' 
 #' @param x design dataframe created by \code{\link{create.dmdf}}
 #' @param formula formula for model in R format
@@ -27,6 +27,7 @@
 #' @param restrict if TRUE, only use design data with Time >= Cohort
 #' @param use.admb if TRUE uses mixed.model.admb for random effects; otherwise mixed.model
 #' @param remove.unused.columns if TRUE, unused columns are removed; otherwise they are left
+#' @param simplify if TRUE simplies real parameter structure for some models; at this time it is not more efficient so ignore
 #' @return create.dm returns a fixed effect design matrix constructed with the design dataframe and the
 #' formula for a single parametre.  It excludes any columns that are all 0. create.dml returns a list with an element for
 #' for each parameter with a sub-list for the fixed effect (fe) and random effects. The re structure depends
@@ -90,7 +91,8 @@ create.dm=function(x, formula, time.bins=NULL, cohort.bins=NULL, age.bins=NULL,
    npar=ncol(mm)
    nrows=nrow(x)
    upper=0
-   dm=Matrix(0,nrow=nrows,ncol=npar)
+#   dm=Matrix(0,nrow=nrows,ncol=npar)
+   dm=NULL
    pieces=floor(npar*nrows/chunk_size+1)
    rows_in_piece=floor(nrows/pieces)
    if(npar*nrows>chunk_size)
@@ -102,14 +104,19 @@ create.dm=function(x, formula, time.bins=NULL, cohort.bins=NULL, age.bins=NULL,
 		  upper=i*rows_in_piece
 		  if(i==1)
 		  {
-			  mm=as(model.matrix(formula,x[lower:upper,,drop=FALSE]),"sparseMatrix") 
-			  dm[lower:upper,]=mm
-		  }
-		  dm[lower:upper,]=as(model.matrix(formula,x[lower:upper,,drop=FALSE]),"sparseMatrix") 
+			  dm=as(model.matrix(formula,x[lower:upper,,drop=FALSE]),"sparseMatrix") 
+#			  dm[lower:upper,]=mm
+		  } else
+		dm=rBind(dm,as(model.matrix(formula,x[lower:upper,,drop=FALSE]),"sparseMatrix"))
+#		dm[lower:upper,]=as(model.matrix(formula,x[lower:upper,,drop=FALSE]),"sparseMatrix") 
 	  }
    }
    if(upper<nrow(x))
-	   dm[(upper+1):nrow(x),]=as(model.matrix(formula,x[(upper+1):nrow(x),,drop=FALSE]),"sparseMatrix")    
+	   if(is.null(dm))
+		   dm=as(model.matrix(formula,x[(upper+1):nrow(x),,drop=FALSE]),"sparseMatrix")
+	   else
+	      dm=rBind(dm,as(model.matrix(formula,x[(upper+1):nrow(x),,drop=FALSE]),"sparseMatrix")) 
+#   dm[(upper+1):nrow(x),]=as(model.matrix(formula,x[(upper+1):nrow(x),,drop=FALSE]),"sparseMatrix")    
    colnames(dm)=colnames(mm)
 #  Remove any unused columns; this is slower but uses less memory
    if(remove.unused.columns)
@@ -125,7 +132,7 @@ create.dm=function(x, formula, time.bins=NULL, cohort.bins=NULL, age.bins=NULL,
 	   return(dm)
    }
 }
-create.dml=function(ddl,model.parameters,design.parameters,restrict=FALSE,chunk_size=1e7,use.admb=FALSE,remove.unused.columns=TRUE)
+create.dml=function(ddl,model.parameters,design.parameters,restrict=FALSE,chunk_size=1e7,use.admb=FALSE,remove.unused.columns=TRUE,simplify=FALSE)
 {
 	dml=vector("list",length=length(model.parameters))
 	names(dml)=names(model.parameters)
@@ -145,11 +152,18 @@ create.dml=function(ddl,model.parameters,design.parameters,restrict=FALSE,chunk_
 		dml[[i]]$fe=create.dm(dd,as.formula(mlist$fix.model),design.parameters[[pn]]$time.bins,
 				design.parameters[[pn]]$cohort.bins,design.parameters[[pn]]$age.bins,chunk_size=chunk_size,model.parameters[[i]]$remove.intercept,
 			    remove.unused.columns=remove.unused.columns)
+		if(simplify)
+		{
+			dml[[i]]$indices=realign.pims(dml[[i]]$fe)
+			dml[[i]]$fe=dml[[i]]$fe[dml[[i]]$indices,,drop=FALSE]
+		} 
 		# if some reals are fixed, assign 0 to rows of dm and then
 		# remove any columns (parameters) that are all 0.
 		if(!is.null(dd$fix)&&any(!is.na(dd$fix)))
 		{
-			dml[[i]]$fe[!is.na(dd$fix),]=0
+#			dml[[i]]$fe[!is.na(dd$fix),]=0
+			zeros=Matrix(rep(as.numeric(is.na(dd$fix)),each=ncol(dml[[i]]$fe)),byrow=T,ncol=ncol(dml[[i]]$fe),nrow=nrow(dml[[i]]$fe))
+			dml[[i]]$fe=dml[[i]]$fe*zeros
 			dml[[i]]$fe=dml[[i]]$fe[,apply(dml[[i]]$fe,2,function(x) any(x!=0)),drop=FALSE]
 		}
 		if(!is.null(mlist$re.model))
@@ -162,4 +176,3 @@ create.dml=function(ddl,model.parameters,design.parameters,restrict=FALSE,chunk_
 	}
 	return(dml)
 }
-
