@@ -113,13 +113,14 @@ mixed.model=function(formula,data,indices=FALSE)
 		  }else
 		  {
 		    sub=strsplit(mlist$re.model[[i]]$sub, "~ ")[[1]][2]
-		    mod=strsplit(mlist$re.model[[i]]$mod, "~ ")[[1]][2]
+		    mod=strsplit(mlist$re.model[[i]]$model, "~ ")[[1]][2]
 		    if(mod=="1") form=formula(paste(c("~", sub), collapse=""))
 		    else form=formula(paste(c("~", mod, ":(", sub, ")"), collapse=""))
 		    dm=model.matrix(form, data)
+        colnames(dm) = paste("(",mod,"|",strsplit(sub, " - 1"),")",c(1:ncol(dm)), sep="")
 		    dm=dm[,colSums(dm)!=0]
 			  re.list[[i]]=Matrix(dm)
-		    names(re.list)[i]=paste(c(mod, "|",strsplit(sub, " -1")), collapse="")
+		    names(re.list)[i]=paste(c(mod, "|",strsplit(sub, " - 1")), collapse="")
 		  }
         }
       }
